# fibooks by Timo Kats
# version 1.0.3
# April 2021
# income_statement.py

import json
import xlsxwriter

class income_statement:
    def __init__(self, company_name):
        # meta data
        self.company_name = company_name
        # fibooks parts
        self.revenue = {}
        self.expenses = {}
        self.income_statement = {}

    # income statement functions

    def add_revenue(self, field, value):
        self.revenue[field] = value

    def add_expense(self, field, value):
        self.expenses[field] = value

    def get_revenues(self):
        value_revenues = 0
        for item in self.income_statement:
            if item == 'revenue':
                for subitem, value in self.revenue.items():
                    value_revenues += value
        return value_revenues

    def get_expenses(self):
        value_expenses = 0
        for item in self.income_statement:
            if item == 'expenses':
                for subitem, value in self.expenses.items():
                    value_expenses += value
        return value_expenses

    def get_netincome(self):
        return self.get_revenues() - self.get_expenses()

    def delete_revenue(self, field):
        del self.revenue[field]

    def delete_expense(self, field):
        del self.expenses[field]

    def empty(self):
        self.revenue = {}
        self.expenses = {}
        self.income_statement = {}

    def make(self):
        self.income_statement['revenue'] = self.revenue
        self.income_statement['expenses'] = self.expenses

    # file import/export functions

    def import_json(self, filename):
        with open(filename) as income_statement:
            data = json.load(income_statement)
            self.revenue = data['revenue']
            self.expenses = data['expenses']

    def export_json(self, filename):
        file = open(filename, 'w+')
        file.write(str(json.dumps(self.income_statement, indent=4)))
        file.close()

    def export_excel(self, filename):
        workbook = xlsxwriter.Workbook(filename)
        worksheet = workbook.add_worksheet()
        worksheet.set_column('A:A', 20)
        worksheet.set_column('B:B', 20)
        bold = workbook.add_format({'bold': True})
        row = 0

        for item in self.income_statement:
            if item == 'revenue':
                worksheet.write(row, 0, 'revenue', bold)
                row += 1
                for sub_item, value in self.revenue.items():
                    worksheet.write(row, 0, sub_item)
                    worksheet.write(row, 1, value)
                    row += 1
            if item == 'expenses':
                worksheet.write(row, 0, 'expenses', bold)
                row += 1
                for sub_item, value in self.expenses.items():
                    worksheet.write(row, 0, sub_item)
                    worksheet.write(row, 1, value)
                    row += 1
        row += 1
        worksheet.write(row, 0, 'net income', bold)
        worksheet.write(row, 1, self.get_netincome())
        workbook.close()

    def export_text(self, filename):
        file = open(filename, 'w+')
        file.write('The income statement of ' + self.company_name)

        for item in self.income_statement:
            if item == 'revenue':
                file.write('\n' + item)
                for subitem, value in self.revenue.items():
                    file.write('\n\t' + subitem + self.get_spaces_print(subitem) + str(value))
            if item == 'expenses':
                file.write('\n' + item)
                for subitem, value in self.expenses.items():
                    file.write('\n\t' + subitem + self.get_spaces_print(subitem) + str(value))
        file.write("\nnet income\t" + self.get_spaces_print('net income') + str(self.get_netincome()))

    # printing to screen functions

    def get_spaces_print(self, item):
        spaces = 30 - len(item)
        return spaces * ' '

    def print(self):
        print('The income statement of', self.company_name)

        for item in self.income_statement:
            if item == 'revenue':
                print(item)
                for subitem, value in self.revenue.items():
                    print('\t', subitem, self.get_spaces_print(subitem), value)
            if item == 'expenses':
                print(item)
                for subitem, value in self.expenses.items():
                    print('\t', subitem, self.get_spaces_print(subitem), value)
        print("\nnet income\t", self.get_spaces_print('net income'), self.get_netincome())



# fibooks by Timo Kats
# version 1.0.3
# April 2021debt
# compute.py

class compute:

    # leverage ratios

    def debt_to_capital_ratio(balance_sheet):
        return round((balance_sheet.get_liabilities() / (balance_sheet.get_equity() + balance_sheet.get_liabilities())), 2)

    def debt_equity_ratio(balance_sheet):
        return round(((balance_sheet.get_liabilities() - balance_sheet.get_field('cash')) / (balance_sheet.get_equity() + balance_sheet.get_liabilities())), 2)

    def MB_ratio(MV_equity, balance_sheet):
        return round(MV_equity / balance_sheet.get_equity(),2)

    # liquidity ratios

    def current_ratio(balance_sheet):
        return round(balance_sheet.get_current_assets() / balance_sheet.get_current_liabilities(), 2)

    def quick_ratio(balance_sheet):
        return round((balance_sheet.get_field('cash') + balance_sheet.get_field('accounts recievable')) / balance_sheet.get_current_liabilities(), 2)

    def cash_ratio(balance_sheet):
        return round(balance_sheet.get_field('cash') / balance_sheet.get_current_liabilities(), 2)

    # other

    def enterprise_value(MV_equity, balance_sheet):
        return MV_equity + balance_sheet.get_liabilities() - balance_sheet.get_field('cash')

    def MV_equity(stocks, stockprice):
        return round(stocks * stockprice, 2)


# PyBooks by Timo Kats
# version 1.0.3
# April 2021
# statement_of_equity.py

import json
import xlsxwriter

class statement_of_equity:
    def __init__(self, company_name):
        print('statement_of_equity will be added in the next update...')


# fibooks by Timo Kats
# version 1.0.3
# April 2021
# __init__.py

import json
import xlsxwriter

try:
    from fibooks.classes.info import info
    from fibooks.classes.balance_sheet import balance_sheet
    from fibooks.classes.income_statement import income_statement
    from fibooks.classes.statement_of_cash_flows import statement_of_cash_flows
    from fibooks.classes.statement_of_equity import statement_of_equity
    from fibooks.classes.compute import compute
except:
    print('fibooks failed to load. please check your installation and version.')


# fibooks by Timo Kats
# version 1.0.3
# April 2021
# statement_of_cash_flows.py

import json
import xlsxwriter

class statement_of_cash_flows:
    def __init__(self, company_name):
        # meta data
        self.company_name = company_name
        # fibooks parts
        self.operating = {}
        self.investing = {}
        self.financing = {}
        self.statement_of_cash_flows = {}

    # statement of cash flows functions

    def add_operating_activity(self, field, value):
        self.operating[field] = value

    def add_investing_activity(self, field, value):
        self.investing[field] = value

    def add_financing_activity(self, field, value):
        self.financing[field] = value

    def get_operating_cash(self):
        value_operating_cash = 0
        for item, value in self.operating.items():
            value_operating_cash += value
        return value_operating_cash

    def get_investing_cash(self):
        value_investing_cash = 0
        for item, value in self.investing.items():
            value_investing_cash += value
        return value_investing_cash

    def get_financing_cash(self):
        value_financing_cash = 0
        for item, value in self.financing.items():
            value_financing_cash += value
        return value_financing_cash

    def get_net_cash(self):
        return self.get_operating_cash() + self.get_investing_cash() + self.get_financing_cash()

    def delete_operating_activity(self, field):
        del self.operating[field]

    def delete_investing_activity(self, field):
        del self.investing[field]

    def delete_financing_activity(self, field):
        del self.financing[field]

    def empty(self):
        self.operating = {}
        self.investing = {}
        self.financing = {}
        self.statement_of_cash_flows = {}

    def make(self):
        self.statement_of_cash_flows['operating'] = self.operating
        self.statement_of_cash_flows['investing'] = self.investing
        self.statement_of_cash_flows['financing'] = self.financing

    # file import/export functions

    def import_json(self, filename):
        with open(filename) as statement_of_cash_flows:
            data = json.load(statement_of_cash_flows)
            self.operating = data['operating']
            self.investing = data['investing']
            self.financing = data['financing']

    def export_json(self, filename):
        file = open(filename, 'w+')
        file.write(str(json.dumps(self.statement_of_cash_flows, indent=4)))
        file.close()

    def export_excel(self, filename):
        workbook = xlsxwriter.Workbook(filename)
        worksheet = workbook.add_worksheet()
        worksheet.set_column('A:A', 20)
        worksheet.set_column('B:B', 20)
        bold = workbook.add_format({'bold': True})
        row = 0

        for item in self.statement_of_cash_flows:
            if item == 'operating':
                worksheet.write(row, 0, 'operating activities', bold)
                row += 1
                for sub_item, value in self.operating.items():
                    worksheet.write(row, 0, sub_item)
                    worksheet.write(row, 1, value)
                    row += 1
            if item == 'investing':
                worksheet.write(row, 0, 'investing activities', bold)
                row += 1
                for sub_item, value in self.investing.items():
                    worksheet.write(row, 0, sub_item)
                    worksheet.write(row, 1, value)
                    row += 1
            if item == 'financing':
                worksheet.write(row, 0, 'financing activities', bold)
                row += 1
                for sub_item, value in self.financing.items():
                    worksheet.write(row, 0, sub_item)
                    worksheet.write(row, 1, value)
                    row += 1
        row += 1
        worksheet.write(row, 0, 'change in cash', bold)
        worksheet.write(row, 1, self.get_net_cash())
        workbook.close()

    def export_text(self, filename):

        file = open(filename, 'w+')

        file.write('The income statement of ' + self.company_name)

        for item in self.statement_of_cash_flows:
            if item == 'operating':
                file.write('\n' + item)
                for subitem, value in self.operating.items():
                    file.write('\n\t' + subitem + self.get_spaces_print(subitem) + str(value))
            if item == 'investing':
                file.write('\n' + item)
                for subitem, value in self.investing.items():
                    file.write('\n\t' + subitem + self.get_spaces_print(subitem) + str(value))
            if item == 'financing':
                file.write('\n' + item)
                for subitem, value in self.financing.items():
                    file.write('\n\t' + subitem + self.get_spaces_print(subitem) + str(value))
        file.write("\nnchange in cash\t" + self.get_spaces_print('change in cash') + str(self.get_net_cash()))

    # printing to screen functions

    def get_spaces_print(self, item):
        spaces = 30 - len(item)
        return spaces * ' '

    def print(self):
        print('The statement of cash flows of', self.company_name)

        for item in self.statement_of_cash_flows:
            if item == 'operating':
                print(item)
                for subitem, value in self.operating.items():
                    print('\t', subitem, self.get_spaces_print(subitem), value)
            if item == 'investing':
                print(item)
                for subitem, value in self.investing.items():
                    print('\t', subitem, self.get_spaces_print(subitem), value)
            if item == 'financing':
                print(item)
                for subitem, value in self.financing.items():
                    print('\t', subitem, self.get_spaces_print(subitem), value)
        print("\nchange in cash\t", self.get_spaces_print('change in cash'), self.get_net_cash())


# fibooks by Timo Kats
# version 1.0.3
# April 2021
# balance_sheet.py

import json
import xlsxwriter

class balance_sheet:
    def __init__(self, company_name):
        # meta data
        self.company_name = company_name
        # fibooks parts
        self.balance_sheet = {}
        self.assets = {}
        self.equity = {}
        self.liabilities = {}
        # subsets
        self.current_assets = {}
        self.longterm_assets = {}
        self.current_liabilities = {}
        self.longterm_liabilities = {}

    # balance sheet functions

    def check_identity(self):
        return self.get_assets() == self.get_equity() + self.get_liabilities()

    def get_assets(self):
        value_assets = 0
        for item in self.balance_sheet:
            if item == 'assets':
                for sub_item, value in self.current_assets.items():
                    value_assets += value
                for sub_item, value in self.longterm_assets.items():
                    value_assets += value
        return round(value_assets, 2)

    def get_equity(self):
        value_equity = 0
        for item in self.balance_sheet:
            if item == 'equity':
                for sub_item, value in self.equity.items():
                    value_equity += value
        return round(value_equity, 2)

    def get_liabilities(self):
        value_liabilities = 0
        for item in self.balance_sheet:
            if item == 'liabilities':
                for sub_item, value in self.current_liabilities.items():
                    value_liabilities += value
                for sub_item, value in self.longterm_liabilities.items():
                    value_liabilities += value
        return round(value_liabilities, 2)

    def get_current_assets(self):
        value_assets = 0
        for item in self.balance_sheet:
            if item == 'assets':
                for sub_item, value in self.current_assets.items():
                    value_assets += value
        return round(value_assets, 2)

    def get_longterm_assets(self):
        value_assets = 0
        for item in self.balance_sheet:
            if item == 'assets':
                for sub_item, value in self.longterm_assets.items():
                    value_assets += value
        return round(value_assets, 2)

    def get_current_liabilities(self):
        value_liabilities = 0
        for item in self.balance_sheet:
            if item == 'liabilities':
                for sub_item, value in self.current_liabilities.items():
                    value_liabilities += value
        return round(value_liabilities, 2)

    def get_longterm_liabilities(self):
        value_liabilities = 0
        for item in self.balance_sheet:
            if item == 'liabilities':
                for sub_item, value in self.longterm_liabilities.items():
                    value_liabilities += value
        return round(value_liabilities, 2)

    def get_field(self, field):
        for item in self.balance_sheet:
            if item == 'assets':
                for sub_item, value in self.current_assets.items():
                    if sub_item == field:
                        return value
                for sub_item, value in self.longterm_assets.items():
                    if sub_item == field:
                        return value
            if item == 'liabilities':
                for sub_item, value in self.current_liabilities.items():
                    if sub_item == field:
                        return value
                for sub_item, value in self.longterm_liabilities.items():
                    if sub_item == field:
                        return value
            if item == 'equity':
                for sub_item, value in self.equity.items():
                    if sub_item == field:
                        return value
        print('field (' + field + ') not found...')
        return 0

    def empty(self):
        # fibooks parts
        self.balance_sheet = {}
        self.assets = {}
        self.equity = {}
        self.liabilities = {}
        # subsets
        self.current_assets = {}
        self.longterm_assets = {}
        self.current_liabilities = {}
        self.longterm_liabilities = {}

    def make(self):
        # set assets
        self.assets['current_assets'] = self.current_assets
        self.assets['longterm_assets'] = self.longterm_assets
        self.balance_sheet['assets'] = self.assets
        # set liabilities
        self.liabilities['current_liabilities'] = self.current_liabilities
        self.liabilities['longterm_liabilities'] = self.longterm_liabilities
        self.balance_sheet['liabilities'] = self.liabilities
        # set equity
        self.balance_sheet['equity'] = self.equity

        if not self.check_identity():
            self.print_warning('make')

    def add_current_asset(self, field, value):
        self.current_assets[field] = value

    def add_longterm_asset(self, field, value):
        self.longterm_assets[field] = value

    def add_current_liability(self, field, value):
        self.current_liabilities[field] = value

    def add_longterm_liability(self, field, value):
        self.longterm_liabilities[field] = value

    def delete_current_asset(self, field):
        del self.current_assets[field]

    def delete_longterm_asset(self, field):
        del self.longterm_assets[field]

    def delete_current_liability(self, field):
        del self.current_liabilities[field]

    def delete_longterm_liability(self, field):
        del self.longterm_liabilities[field]

    def add_equity(self, field, value):
        self.equity[field] = value

    # file import/export functions

    def import_json(self, filename):
        with open(filename) as balance_sheet:
            data = json.load(balance_sheet)
            self.current_assets = data['assets']['current_assets']
            self.longterm_assets = data['assets']['longterm_assets']
            self.current_liabilities = data['liabilities']['current_liabilities']
            self.longterm_liabilities = data['liabilities']['longterm_liabilities']
            self.equity = data['equity']

    def export_json(self, filename):
        if not self.check_identity():
            self.print_warning('export_json')
        file = open(filename, 'w+')
        file.write(str(json.dumps(self.balance_sheet, indent=4)))
        file.close()

    def export_excel(self, filename):
        workbook = xlsxwriter.Workbook(filename)
        worksheet = workbook.add_worksheet()
        worksheet.set_column('A:A', 20)
        worksheet.set_column('B:B', 20)
        bold = workbook.add_format({'bold': True})
        row = 0

        if not self.check_identity():
            self.print_warning('export_excel')

        for item in self.balance_sheet:
            if item == 'assets':
                worksheet.write(row, 0, 'assets', bold)
                row += 1
                worksheet.write(row, 0, 'current assets', bold)
                for sub_item, value in self.current_assets.items():
                    row += 1
                    worksheet.write(row, 0, sub_item)
                    worksheet.write(row, 1, value)
                row += 1
                worksheet.write(row, 0, 'longterm assets', bold)
                for sub_item, value in self.longterm_assets.items():
                    row += 1
                    worksheet.write(row, 0, sub_item)
                    worksheet.write(row, 1, value)
                worksheet.write(row + 1, 1, self.get_assets())
            row += 1
            if item == 'liabilities':
                worksheet.write(row, 0, 'liabilities', bold)
                row += 1
                worksheet.write(row, 0, 'current liabilities', bold)
                for sub_item, value in self.current_liabilities.items():
                    row += 1
                    worksheet.write(row, 0, sub_item)
                    worksheet.write(row, 1, value)
                row += 1
                worksheet.write(row, 0, 'longterm liabilities', bold)
                for sub_item, value in self.longterm_liabilities.items():
                    row += 1
                    worksheet.write(row, 0, sub_item)
                    worksheet.write(row, 1, value)
                worksheet.write(row + 1, 1, self.get_liabilities())
            row += 1
            if item == 'equity':
                worksheet.write(row, 0, 'equity', bold)
                for sub_item, value in self.equity.items():
                    row += 1
                    worksheet.write(row, 0, sub_item)
                    worksheet.write(row, 1, value)
                worksheet.write(row + 1, 1, self.get_equity())
        workbook.close()

    def export_text(self, filename):

        file = open(filename, 'w+')

        if not self.check_identity():
            self.print_warning('export_text')

        file.write("The balance sheet of: " + self.company_name)
        for item in self.balance_sheet:
            if item == 'assets':
                file.write("\nassets\n")
                file.write("\tcurrent assets\n")
                for sub_item, value in self.current_assets.items():
                    file.write("\n\t\t" + sub_item + self.get_spaces_text(sub_item) + str(value))
                file.write("\n\tlong-term assets\n")
                for sub_item, value in self.longterm_assets.items():
                    file.write("\n\t\t" + sub_item + self.get_spaces_text(sub_item) + str(value))
                file.write("\n\t\t" + self.get_spaces_text_equity("") + "___+")
                file.write(
                    "\n\t" + "total assets" + self.get_spaces_text_equity("total assets") + str(self.get_assets()))
            if item == 'liabilities':
                file.write("\nliabilities\n")
                file.write("\tcurrent liabilities\n")
                for sub_item, value in self.current_liabilities.items():
                    file.write("\n\t\t" + sub_item + self.get_spaces_text(sub_item) + str(value))
                file.write("\n\tlong-term liabilities\n")
                for sub_item, value in self.longterm_liabilities.items():
                    file.write("\n\t\t" + sub_item + self.get_spaces_text(sub_item) + str(value))
                file.write("\n\t\t" + self.get_spaces_text_equity("") + "___+")
                file.write("\n\t" + "total liabilities" + self.get_spaces_text_equity("total liabilities") + str(
                    self.get_liabilities()))
            if item == 'equity':
                file.write("\nequity\n")
                for sub_item, value in self.equity.items():
                    file.write("\n\t" + sub_item + self.get_spaces_text_equity(sub_item) + str(value))
                file.write("\n\t\t" + self.get_spaces_text_equity("") + "___+")
                file.write(
                    "\n\t" + "total equity" + self.get_spaces_text_equity("total equity") + str(self.get_equity()))

    # printing to screen functions

    def print_warning(self, function):
        print("error in", function, ": balance sheet identity not correct!")

    def get_spaces_text(self, item):
        spaces = 40 - len(item)
        return spaces * ' '

    def get_spaces_text_equity(self, item):
        spaces = 40 - len(item)
        return (spaces * ' ') + '\t'

    def get_spaces_print(self, item):
        spaces = 30 - len(item)
        return spaces * ' '

    def get_spaces_print_equity(self, item):
        spaces = 30 - len(item)
        return (spaces * ' ') + '\t'

    def print(self):

        if not self.check_identity():
            self.print_warning('print')

        print("The balance sheet of: ", self.company_name)
        for item in self.balance_sheet:
            if item == 'assets':
                print("\nassets")
                print("\tcurrent assets")
                for sub_item, value in self.current_assets.items():
                    print("\t\t", sub_item, self.get_spaces_print(sub_item), value)
                print("\tlong-term assets")
                for sub_item, value in self.longterm_assets.items():
                    print("\t\t", sub_item, self.get_spaces_print(sub_item), value)
                print("\t\t", self.get_spaces_print_equity(""), "___+")
                print("\t", "total assets", self.get_spaces_print_equity("total assets"), self.get_assets())
            if item == 'liabilities':
                print("\nliabilities")
                print("\tcurrent liabilities")
                for sub_item, value in self.current_liabilities.items():
                    print("\t\t", sub_item, self.get_spaces_print(sub_item), value)
                print("\tlong-term liabilities")
                for sub_item, value in self.longterm_liabilities.items():
                    print("\t\t", sub_item, self.get_spaces_print(sub_item), value)
                print("\t\t", self.get_spaces_print_equity(""), "___+")
                print("\t", "total liabilities", self.get_spaces_print_equity("total liabilities"),
                      self.get_liabilities())
            if item == 'equity':
                print("\nequity")
                for sub_item, value in self.equity.items():
                    print("\t", sub_item, self.get_spaces_print_equity(sub_item), value)
                print("\t\t", self.get_spaces_print_equity(""), "___+")
                print("\t", "total equity", self.get_spaces_print_equity("total equity"), self.get_equity())



from setuptools import setup, find_packages

classifiers = [
    'Development Status :: 4 - Beta',
    'Intended Audience :: Financial and Insurance Industry',
    'License :: OSI Approved :: MIT License',
    'Natural Language :: English',
    'Operating System :: Microsoft :: Windows :: Windows 10',
    'Programming Language :: Python :: 3'
]

setup(

    name='fibooks',
    version='1.0.4',
    description='Python library for financial statement analysis. The full documentation can be found at: https://github.com/TimoKats/fibooks',
    long_description='Python library for financial statement analysis. The full documentation can be found at: https://github.com/TimoKats/fibooks',
    author='Timo Kats',
    author_email='tpakats@gmail.com',
    license='MIT',
    classifiers=classifiers,
    keywords='finance',
    packages=['fibooks', 'fibooks.classes'],
    install_requires=['xlsxwriter']

)


# fibooks by Timo Kats
# version 1.0.3
# April 2021
# info.py

class info:
    def get_version():
        print("version:         1.0.3")
        print("documentation:   https://github.com/TimoKats/fibooks")
        print("made by:         Timo Kats")

    def secret_function():
        print('there are no easter eggs here...go away')